function MemoryGame() {
    var self = this;

    self.currentCard = null;
    self.canSelect = true;
    self.solved = 0;
    self.numberToSolve = 20;

    self.cards = ['hairy', 'schnitzel', 'slinky', 'bottomley', 'zachary', 'hercules', 'bitzer', 'scarface', 'muffin'];

    //Set up hammer interaction
    var gameboard = $(".game--memory .cards");


    var engine,
        docStyle = document.documentElement.style;
    if (window.opera && Object.prototype.toString.call(opera) === '[object Opera]') {
        engine = 'presto';
    } else if ('MozAppearance' in docStyle) {
        engine = 'gecko';
    } else if ('WebkitAppearance' in docStyle) {
        engine = 'webkit';
    } else if (typeof navigator.cpuClass === 'string') {
        engine = 'trident';
    }

    var vendorPrefix = {
        trident: 'ms',
        gecko: 'Moz',
        webkit: 'Webkit',
        presto: 'O'
    }[engine];

    var transformProperty = vendorPrefix + "Transform";

    //$(window).on("resize", resize)

    function resize(){

        if($(".game--memory").hasClass("show") && $(".cards").hasClass("show")){

            //Calculate width of tiles
            var linecount;
            switch(gameboard.data("level")){
                case 1:
                    linecount = 4;
                    break;
                case 2:
                    linecount = 4;
                    break;
                case 3:
                    linecount = 6;
                    break;                   

            }
            var width = linecount * $(".card").outerWidth(true);

            //Calculate scale
            var scale = $(window).height() * .75 / gameboard.height();

            gameboard.css(transformProperty, "scale(" + scale + ")");


        }
    }

    self.start = function () {
        self.solved = 0;

        $('.game-lobby').addClass('hide');
        $('.game--memory').addClass('show');


        $('.dialog--memory-results').hide();
        $('.dialog--memory-instructions').show();

        gameboard.removeClass("show");
        
        //$('#memoryGame .active').removeClass('active');
      
        
        //$('#memoryGame .card').css('opacity', 1);

        //$('#memoryGame .active').removeClass('active');
        
    }

    self.showBoard = function () {

        $('.dialog--memory-instructions').hide();
        gameboard.addClass("show");

        var level = parseInt($(this).attr('data-level'));

        //$('#memoryGame .cards').removeClass('difficulty--1 difficulty--2 difficulty--3').addClass('difficulty--' + level);

        var cardsInGame = [];

        switch (level) {
            case 1:
                cardsInGame.push(self.cards[0]);
                cardsInGame.push(self.cards[1]);
                cardsInGame.push(self.cards[2]);
                cardsInGame.push(self.cards[3]);
                break;

            case 2:
                cardsInGame.push(self.cards[0]);
                cardsInGame.push(self.cards[1]);
                cardsInGame.push(self.cards[2]);
                cardsInGame.push(self.cards[3]);
                cardsInGame.push(self.cards[4]);
                cardsInGame.push(self.cards[5]);
                break;

            case 3:
                cardsInGame = self.cards;
                break;
        }

        self.numberToSolve = cardsInGame.length;

        /* Consider modifying this to use moustache templates */
        var template = '<div class="card {character}" data-character="{character}">';
        template += '<div class="flipper">';
        template += '<div class="back"></div>';
        template += '<div class="front"></div>';
        template += '</div>';
        template += '</div>';

        
        gameboard.html('');
        gameboard.removeClass('difficulty--1 difficulty--2 difficulty--3');
        gameboard.addClass('difficulty--' + level).data("level", level);

        for (var i = 0; i < cardsInGame.length; i++) {        
            $( template.replace('{character}', cardsInGame[i]).replace('{character}', cardsInGame[i]) ).appendTo(gameboard);        
            $( template.replace('{character}', cardsInGame[i]).replace('{character}', cardsInGame[i]) ).appendTo(gameboard);
        }

        self.reorder();
        $(".card .front").hammer().on('tap', card_click);
    
        /* Add line breaks to even up the lines */
        switch (level) {
            case 1:
                $(".cards .card:nth-child(4n)").after("<br>");
                break;
            case 2:
                $(".cards .card:nth-child(4n)").after("<br>");
                break;
            case 3:
                $(".cards .card:nth-child(6n)").after("<br>");
                break;
        }

        $(".dialog--memory-instructions").hide();
        gameboard.addClass("show");
    }

    self.reorder = function () {
        var grp = $(".game--memory .cards").children();
        var cnt = grp.length;

        var temp, x;
        for (var i = 0; i < cnt; i++) {
            temp = grp[i];
            x = Math.floor(Math.random() * cnt);
            grp[i] = grp[x];
            grp[x] = temp;
        }
        $(grp).remove();
        $(".game--memory .cards").append($(grp));
    }

    //Private Methods
    var card_click = function() {
        if (!self.canSelect) {
            return false;
        }
        
        $(this).parent().parent().addClass('active');

        if (self.currentCard === null && self.canSelect) {
            self.currentCard = $(this).parent().parent().attr('data-character');
            return false;
        }

        var character = $(this).parent().parent().attr('data-character');

        if (character === self.currentCard) {
            setTimeout(function () {
                $('#memoryGame div.cards .' + character).css('opacity', 0).addClass('solved');
                self.canSelect = true;
                self.currentCard = null;
                self.solved++;

                if (self.solved === self.numberToSolve) {
                    gameboard.removeClass("show");
                    $('.dialog--memory-results').show();
                }
            }, 750);
        } else {
            self.canSelect = false;
            setTimeout(function () {
                $('#memoryGame .active').removeClass('active');
                self.currentCard = null;
                self.canSelect = true;
            }, 750);
        }
    };

    self.getRandom = function (min, max) {
        return Math.random() * (max - min) + min;
    };


    //Initialise interaction
    $('.button--memory-start').hammer().on('tap', self.showBoard);
    $('.button--memory-playagain').hammer().on('tap', self.start);
}

$(function () {
    games.memory = new MemoryGame();
});